/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.disk.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.disk.domain.CourseAuditRecord;
import vip.xiaonuo.disk.domain.CourseInfo;
import vip.xiaonuo.disk.dto.courseauditrecord.CourseAuditRecordAddParam;
import vip.xiaonuo.disk.dto.courseauditrecord.CourseAuditRecordEditParam;
import vip.xiaonuo.disk.dto.courseauditrecord.CourseAuditRecordIdParam;
import vip.xiaonuo.disk.dto.courseauditrecord.CourseAuditRecordPageParam;
import vip.xiaonuo.disk.dto.shareGroupuser.ShareGroupUser;
import vip.xiaonuo.disk.mapper.CourseAuditRecordMapper;
import vip.xiaonuo.disk.service.CourseAuditRecordService;
import vip.xiaonuo.sys.modular.user.entity.SysUser;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * 课程审核记录表Service接口实现类
 *
 * @author honorfire
 * @date  2025/06/20 14:58
 **/
@Service
public class CourseAuditRecordServiceImpl extends ServiceImpl<CourseAuditRecordMapper, CourseAuditRecord> implements CourseAuditRecordService {

    @Resource
    private CourseAuditRecordMapper courseAuditRecordMapper;

    @Override
    public Page<CourseAuditRecord> page(CourseAuditRecordPageParam courseAuditRecordPageParam) {
        QueryWrapper<CourseAuditRecord> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(courseAuditRecordPageParam.getSortField(), courseAuditRecordPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(courseAuditRecordPageParam.getSortOrder());
            queryWrapper.orderBy(true, courseAuditRecordPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(courseAuditRecordPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(CourseAuditRecord::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Override
    public List<CourseAuditRecord> queryCourseAuditRecordList(Map param) {
        QueryWrapper<CourseAuditRecord> queryWrapper = new QueryWrapper<>();
        if (ObjectUtil.isNotEmpty(param.get("verifyStatus"))) {
            queryWrapper.lambda().eq(CourseAuditRecord::getVerifyStatus, param.get("verifyStatus"));
        }
        if (ObjectUtil.isNotEmpty(param.get("courseId"))) {
            queryWrapper.lambda().eq(CourseAuditRecord::getCourseId, param.get("courseId"));
        }
        return this.list(queryWrapper);
    }

    /**
     * 课程审核记录-批量添加
     *
     * @author honorfire
     * @date  2025/06/18 14:16
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addBatch(List<CourseAuditRecord> courseAuditRecordList)
    {
        this.saveBatch(courseAuditRecordList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(CourseAuditRecordAddParam courseAuditRecordAddParam) {
        CourseAuditRecord courseAuditRecord = BeanUtil.toBean(courseAuditRecordAddParam, CourseAuditRecord.class);
        this.save(courseAuditRecord);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CourseAuditRecord addOne(CourseAuditRecord courseAuditRecord) {
        this.save(courseAuditRecord);
        return courseAuditRecord;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(CourseAuditRecordEditParam courseAuditRecordEditParam) {
        CourseAuditRecord courseAuditRecord = this.queryEntity(courseAuditRecordEditParam.getId());
        BeanUtil.copyProperties(courseAuditRecordEditParam, courseAuditRecord);
        this.updateById(courseAuditRecord);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CourseAuditRecord editOne(CourseAuditRecord courseAuditRecord) {
        this.updateById(courseAuditRecord);
        return courseAuditRecord;
    }

    /**
     * 课程审核记录-批量更新
     *
     * @author honorfire
     * @date  2025/06/18 14:16
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void editBatch(List<CourseAuditRecord> courseAuditRecordList)
    {
        this.updateBatchById(courseAuditRecordList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<CourseAuditRecordIdParam> courseAuditRecordIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(courseAuditRecordIdParamList, CourseAuditRecordIdParam::getId));
    }

    @Override
    public CourseAuditRecord detail(CourseAuditRecordIdParam courseAuditRecordIdParam) {
        return this.queryEntity(courseAuditRecordIdParam.getId());
    }

    @Override
    public CourseAuditRecord queryEntity(String id) {
        CourseAuditRecord courseAuditRecord = this.getById(id);
        if(ObjectUtil.isEmpty(courseAuditRecord)) {
            throw new CommonException("课程审核记录表不存在，id值为：{}", id);
        }
        return courseAuditRecord;
    }

    @Override
    public Page<Map<String,Object>> queryList(Map param)
    {
        return courseAuditRecordMapper.queryList(param,CommonPageRequest.defaultPage());
    }

    @Override
    public Page<Map<String,Object>> queryResourceCentreList(Map param)
    {
        return courseAuditRecordMapper.queryResourceCentreList(param,CommonPageRequest.defaultPage());
    }


    @Override
    public Map<String,Object> queryInfo(Map param)
    {
        return courseAuditRecordMapper.queryInfo(param);
    }

    @Override
    public Map<String,Object> queryRecentlyRecord(Map param)
    {
        return courseAuditRecordMapper.queryRecentlyRecord(param);
    }

}
